--[[
    File: tuf_aura_db.lua
    Author: Tuck
    Description: This file contains the management of the database of auras. 

]]

--[[
    Aura Power:

    low:    0 - 99
    medium: 100 - 9999
    high:   1000+
]]

--[[
    Aura Groups (Debuffs):

    - stun: stun debuff
    - fear: fear debuff
    - mezz: mesmerize debuff (loss of control, breaks on damage)
    - silence: debuff preventing casing of spells
    - antiheal: debuff reducing healing received
    - slow: debuff reducing movement speed
    - root: debuff preventing movement
    - dot: magical damage over time debuff
    - bleed: physical damage over time debuff
    
    - power_friend: power if the aura is found on a friend
    - power_enemy: power if the aura is found on an enemy
    - power_self: power if the aura is found on self
]]


local tuf_aura_db = TuckUF.aura_db

tuf_aura_db.aura_configs = {"self", "enemy", "friend"}
tuf_aura_db.aura_options = {["power"] = 0, ["group"] = "", ["is_special"] = false}

local aura_configs = tuf_aura_db.aura_configs
local aura_options = tuf_aura_db.aura_options

local TRACE = function(msg) ChatFrame1:AddMessage(msg) end


--[[
    Returns the information table for the given aura
]]
local empty_table = {}
function TuckUF.aura_db:GetAuraInfo(aura_name)
    return tuf_aura_db.auras[aura_name] or empty_table
end

--[[
    Returns the suffix for the given blizzard unit id
]]
function TuckUF.aura_db:GetOptionSuffix(unit_id)
    local suffix = ""
    if unit_id == "player" then
        suffix = "_self"
    elseif UnitIsFriend(unit_id,"player") then
        suffix = "_friend"
    else
        suffix = "_enemy"
    end
    
    return suffix
end

--[[
    Sets the info for the given aura
    Warning: Call with . and not :
]]
function TuckUF.aura_db.SetAuraInfo(aura_name, aura_info)
    -- reset old data
    tuf_aura_db.auras[aura_name] = {}
    local aura_data = tuf_aura_db.auras[aura_name]
    
    -- copy new data
    if aura_info then
        for option_name, option_value in pairs(aura_info) do
            aura_data[option_name] = option_value
        end
    end
    
    -- fill in missing data
    tuf_aura_db:CompleteAuraInfo(aura_name, aura_data, true)
end

--[[
    Adds the info to the aura info, overwriting existing data
    Warning: Call with . and not :
]]
function TuckUF.aura_db.AddAuraInfo(aura_name, aura_info)
    -- make sure the data exists
    local data = tuf_aura_db.auras[aura_name]
    if data == nil then
        TRACE("Missing data for aura: "..aura_name)
        tuf_aura_db.SetAuraInfo(aura_name, aura_info)
        return
    end
    
    -- process general config
    tuf_aura_db:CompleteAuraInfo(aura_name, aura_info, false)

    -- copy new data
    for option_name, option_value in pairs(aura_info) do
        data[option_name] = option_value
    end
end

--[[
    Adds missing values to the given table. 
    aura_info: table with the options
    add_missing: if true, all options that are completely missing will be set to their default values
]]
function TuckUF.aura_db:CompleteAuraInfo(aura_name, aura_info, add_missing)
    if aura_info == nil then 
        TRACE("Missing data for aura: "..aura_name)
        return 
    end
    
    for option_name, option_default in pairs(aura_options) do          
        for j, config_name in ipairs(aura_configs) do
        
            local complete_name = option_name.."_"..config_name
            
            -- if option/config pair is missing
            if aura_info[complete_name] == nil then
                if aura_info[option_name] ~= nil then 
                    -- replicate general option to all configs
                    aura_info[complete_name] = aura_info[option_name] 
                else
                    -- set missing value if desired
                    if add_missing then
                        aura_info[complete_name] = option_default
                    end
                end
            end
            
        end
    end  
end

--[[

]]
function TuckUF.aura_db:AddDBInitializer(initializer)
    tuf_aura_db.db_initializers = tuf_aura_db.db_initializers or {}
    table.insert(tuf_aura_db.db_initializers, initializer)
end

--[[

]]
function TuckUF.aura_db:InitDB()
    tuf_aura_db.auras = {}
    for i, initializer in ipairs(tuf_aura_db.db_initializers) do
        initializer()
    end
end

